'''
Created on 2013-03-23

@author: Jackie
'''

class DepNode:
    governor = -1
    dependent = -1
    reltype = ''
    
    def __init__(self):
        self.governor = -1
        self.dependent = -1
        self.reltype = ''
        
        
class DepTree:
    nodes = {}
    head = -1
    
    def __init__(self):
        self.nodes = {} # key is the dependent, value is a list
        self.head = -1
    def dependents_of(self, governor):
        '''
        Returns the immediate dependents of governor.
        '''
        return [node.dependent for dep_id, node in self.iternodes() 
                if node.governor == governor]
    def iternodes(self):
        '''
        Iterator over all the (tokennum, nodes) pairs.
        '''
        for dep_id, nodes in self.nodes.iteritems():
            for node in nodes:
                yield (dep_id, node)
    def span_of(self, head_node_id):
        '''
        Returns the start and end of the phrase headed by head_node.
        '''
        children = [child for child in self.dependents_of(head_node_id) if child != head_node_id]
        # base case
        if len(children) == 0:
            return (head_node_id, head_node_id)
        else:
            spans = [self.span_of(child) for child in children]
            start = min(head_node_id, *[x[0] for x in spans])
            end = max(head_node_id, *[x[1] for x in spans])
            return (start, end)
        
    def add(self, dep_id, node):
        '''
        Adder to nodes.
        '''
        list_nodes = self.nodes.setdefault(dep_id, [])
        list_nodes.append(node)
                
class Document:
    """
    A representation of a document, preprocessed by CoreNLP.
    """
    num = -1
    sents = {}
    corefs = []
    docid = ''
    
    def __init__(self):
        self.num = -1
        self.sents = {}
        self.corefs = []
        self.docid = '' # optional element
        # optional element: self.doctype
        
    def at(self, sentnum):
        return self.sents.get(sentnum)

class Sentence:
    """
    A representation of a sentence with its parse, preprocessed by CoreNLP.
    """
    num = -1
    tokens = {}
    parse = DepTree()
    
    def __init__(self):
        self.num = -1
        self.tokens = {}
        self.parse = DepTree()
    def at(self, tokennum):
        return self.tokens.get(tokennum)
    def text(self):
        '''
        Returns a text string of the words in the sentence.
        '''
        keys = sorted(self.tokens.keys())
        sent = ' '.join([self.tokens[key].word for key in keys])
        sent = sent.replace(' ,' , ',')
        sent = sent.replace(' .' , '.')
        sent = sent.replace(' !' , '!')
        sent = sent.replace(' ?' , '?')
        sent = sent.replace('$ ' , '$')
        sent = sent.replace(' :' , ':')
        sent = sent.replace(' ;' , ';')
        sent = sent.replace(' %' , '%')
        sent = sent.replace(' \' ', '\' ')
        sent = sent.replace(' \'s' , '\'s')
        sent = sent.replace(' \'m' , '\'m')
        sent = sent.replace(' \'ve' , '\'ve')
        sent = sent.replace(' \'re' , '\'re')
        sent = sent.replace(' \'d' , '\'d')
        sent = sent.replace(' \'ll' , '\'ll')
        sent = sent.replace('`` ' , '"')
        sent = sent.replace(' \'\'' , '"')
        sent = sent.replace(' n\'t', 'n\'t')
        sent = sent.replace(' _ ', ' -- ')
        sent = sent.replace('-LRB- ', '(')
        sent = sent.replace(' -RRB-', ')')
        sent = sent.replace('` ' , '\'')
        sent = sent.replace(' can not ' , ' cannot ')
        return sent
        
    def real_word_count(self):
        return len(self.text().split())
    
class Token:
    num = -1
    word = ''
    lemma = ''
    pos = ''
    ner = ''


class CorefChain:
    def __init__(self):
        self.mentions = []
        
class CorefMention:
    def __init__(self):
        self.representative = False
        self.sentence = -1
        self.start = -1
        self.end = -1
        self.head = -1
